#!/usr/bin/env python
# -*- coding: utf-8 -*-

# A script to import your byteflow blog to python

__author__ =  "Jocelyn Delalande - http://jocelyn.delalande.fr"
__licence__ =  "WTFPL"

#
# Features:
#  - import blog global metadatas
#  - import articles with MarkDown markup
#  - import images with propper URL handling
#
# Non-features (will not be implemented):
#  - import commnents
#  - other markups than MarkDown
#  - categories
#  - internal links between articles
#  - URLs
#
# We assume that byteflow refers to the local images via something like
#   ![découpe boitier](/media/DSCN0084.JPG)
# "/media/" referring to the folder mentioned as the third arg on the CLI.
# If not, customize BYTEFLOW_IMAGES_URL.
#
# We assume that you want to put your imported images in content/images, if not,
# customize PELICAN_IMAGES_DIR

import sys
import re
import os
import os.path
import shutil

import sqlite3

con = None

BYTEFLOW_IMAGES_URL="/media"

ARTICLE_TEMPLATE = \
"""Title: %(name)s
Date: %(date)s
Tags: %(tags)s
Slug: %(slug)s
Author: %(first_name)s

%(text)s
"""

PELICAN_IMAGES_DIR = "images"

R_BYTEFLOW_LOCAL_IMG = re.compile(
    r'!\[(?P<legend>.+)\]\(%s/(?P<filename>.+)\)' % BYTEFLOW_IMAGES_URL)

def usage():
    err("USAGE:\n  byteflow-to-pelican.py <sqlite_path> <pelican_basedir> [byteflow_images_dir]")

def err(msg):
    sys.stderr.write('%s\n' % msg)

def fetch_articles(db_file_path):
    ARTICLES_TABLE = 'blog_post'
    SQL = 'SELECT name, slug, date, author.first_name, is_draft, tags, text from blog_post JOIN auth_user AS author ON blog_post.author_id = author.id'
    try:
        con = sqlite3.connect(db_file_path)
        con.row_factory = sqlite3.Row
        cur = con.cursor()
        cur.execute(SQL)
        rows = cur.fetchall()
        for row in rows:
            yield row

    except sqlite3.Error, e:
        err("Error %s:" % e.args[0])
        sys.exit(1)

    finally:
        if con:
            con.close()

def make_pelican_article(byteflow_db_row, articles_path):
    row = byteflow_db_row
    filename = os.path.join(articles_path,'%s.md' % row['slug'])

    f = open(filename, 'w')
    data = dict(byteflow_db_row)

    if data['is_draft']:
        f.write('Status: draft\n')

    content = ARTICLE_TEMPLATE % data
    f.write(content.encode('utf-8'))
    f.close()
    return filename

def copy_images(byteflow_images_path, pelican_images_path):
    imported = []
    for f in os.listdir(byteflow_images_path):
        orig = os.path.join(byteflow_images_path, f)
        dest = os.path.join(pelican_images_path, f)
        print "importing %s -> %s" % (orig, dest)

        shutil.copy2(orig, dest)
        imported.append(dest)

    return imported

if __name__ == '__main__':
    if len(sys.argv) < 3:
        usage()
        sys.exit(4)
    else:
        SQLITE_PATH = sys.argv[1]
        PELICAN_PATH = sys.argv[2]
        ARTICLES_PATH = os.path.join(PELICAN_PATH, 'content')

        if not os.path.exists(ARTICLES_PATH)\
                or not os.path.isdir(ARTICLES_PATH):
            err('%s is not a valid pelican path (does not '\
                                 +'contain a "content" folder).')
            sys.exit(5)

        elif not os.access(ARTICLES_PATH, os.W_OK):
            err('%s is not writable, abording.' % ARTICLES_PATH)
            sys.exit(6)

    print "IMPORTING ARTICLES"
    imported_articles = []
    for article in fetch_articles(SQLITE_PATH):
        print '  importing "%s"' % article['name']
        article_filename = make_pelican_article(article, ARTICLES_PATH)
        imported_articles.append(article_filename)

    print "\nSuccessfuly imported %i articles" % len(imported_articles)

    print "\nIMPORTING IMAGES"
    if len(sys.argv) > 3:
        BYTEFLOW_IMAGES_PATH = sys.argv[3]
        PELICAN_IMAGES_PATH = os.path.join(PELICAN_PATH, 'content', PELICAN_IMAGES_DIR)

        if not os.path.exists(PELICAN_IMAGES_PATH):
            os.mkdir(PELICAN_IMAGES_PATH)

        imported_img = copy_images(BYTEFLOW_IMAGES_PATH, PELICAN_IMAGES_PATH)

        print "\nSuccessfuly imported %i images" % len(imported_img)

    else:
        print "No byteflow images dir specified, skipping images import"

    print "\nREWRITING IMAGES URLS in imported articles"

    def rewrite_urls(m):
        v = m.groupdict()
        return '![%s](|filename|/%s/%s)' % (v['legend'], PELICAN_IMAGES_DIR, v['filename'])

    for filename in imported_articles:
        original = open(filename).read()
        mangled = re.sub(R_BYTEFLOW_LOCAL_IMG, rewrite_urls, original)
        if original != mangled:
            open(filename, 'w').write(mangled)
            print "  Rebuilt images URLs in %s" % filename


